/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui;

import isac.gui.worksheet.Worksheet;
import isac.objectmanager.OManager;
import isac.session.SDialog;
import isac.util.Formula;
import isac.util.ModSpec;
import isac.util.ModSpecFormulaListEl;
import isac.util.ModSpecIdEl;
import isac.wsdialog.DGuide;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSplitPane;

/**
 * Main window application for the ISAC system.
 * Responsible for showing the user all requested information and for the
 * interaction with the user.
 * 
 * @author Mario Hochreiter
 * @version 0.1
 *
 */
public class WindowApplication extends JFrame {
  /** Local settings for the user */
  private Locale locale;
  
  /** Container for the internal frames, which will contain the worksheets */
  private JDesktopPane worksheetDesktopPane;
  
  /** Top container that contains all elements of this frame */
  private JSplitPane splitPane;    
  
  /** List that holds all internal frames (worksheets)*/
  private List worksheets;
  
  /** Required to logon a user and to obtain the <code>WSDialog</code> */
  private SDialog sessionDialog;
  
  /** Unique Session ID the current user. */
  private String sessionID;
  
  private DGuide dialog;
  
  /** 
   * Panel that displays the different "browseable" hierarchies (e.g. the
   * example Browser).
   */
  private BrowserPanel browserPanel;
  
  private Worksheet ws;
    
  public WindowApplication() {
    this.initRemoteObjects();
    locale = Locale.getDefault();
    
    this.setSize(new Dimension(1024, 768));
    Point screenCenter = this.screenCenter();
    this.setLocation(screenCenter);
    
    Container contentPane = this.getContentPane();
    this.buildMenuBar();
    worksheets = new ArrayList();
    
    this.createWorksheetPane();
    this.createBrowserPanel();
    this.buildSplitPane();

    contentPane.add(splitPane);
    
    // TODO remove: only for Testing purposes at the moment
    JInternalFrame internalFrame = new JInternalFrame("TestWorksheet", 
                                                      true, false, true);              
    internalFrame.getContentPane().setLayout(new BorderLayout());
    ws = new Worksheet(dialog);
    internalFrame.getContentPane().add(ws, BorderLayout.CENTER);
    internalFrame.pack();
    internalFrame.setVisible(true);
    this.addInternalFrame(internalFrame);
 
    internalFrame = new JInternalFrame("ModSpecPanel", true, false, true);
    internalFrame.getContentPane().setLayout(new BorderLayout());
    internalFrame.getContentPane().add(new ModSpecPanel(this, locale), 
                                       BorderLayout.CENTER);
    this.addInternalFrame(internalFrame);
    internalFrame.pack();
    internalFrame.setVisible(true);
    
    JFrame.setDefaultLookAndFeelDecorated(true);
    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    this.show();
  }
  
  /**
   * Initialize the remote isac objects <code>ObjectManager</code> and
   * <code>SessionDialog</code>.
   */
  private void initRemoteObjects() {
    try {
      RMIClient client = new RMIClient("localhost", 1099);
      OManager objectManager = client.getObjectManager();
      sessionDialog = objectManager.getSessionDialog();
      Vector sessionIDS = sessionDialog.login("test", "test");
      sessionID = (String)sessionIDS.get(0);
      dialog = client.getDialogGuide();
    } catch (AccessException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (NotBoundException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  private void buildMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    // TODO localisation
    JMenu fileMenu = new JMenu("File");
    JMenuItem quitItem = new JMenuItem("Quit");
    fileMenu.add(quitItem);
    menuBar.add(fileMenu);    
    this.setJMenuBar(menuBar);
  }
  
  private void createBrowserPanel() {
    browserPanel = new BrowserPanel();
  }
  
  private void createWorksheetPane() {
    worksheetDesktopPane = new JDesktopPane();
    Dimension d = new Dimension();
    d.setSize(this.getWidth()*0.60, 0.0);
    worksheetDesktopPane.setMinimumSize(d);
    // "ghost frame" dragging; very fast but weired for some user
    worksheetDesktopPane.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE); 
  }
  
  private void buildSplitPane() {
    splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false);
    splitPane.setLeftComponent(browserPanel);
    splitPane.setRightComponent(worksheetDesktopPane);
    splitPane.setOneTouchExpandable(true);
    splitPane.setDividerLocation(250);    
  }
  
  private Point screenCenter() {
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int midX = screenSize.width / 2 - this.getWidth() / 2; 
    int midY = screenSize.height / 2 - this.getHeight() / 2;
    return new Point(midX, midY);
  }
 
  private void addInternalFrame(JInternalFrame frame) {
    worksheetDesktopPane.add(frame);
  }
  
  public void createModSpec(String given, String where, String find, 
                            String theory, String method, String problem, 
                            String relate) {
    
    // TODO: !!! alter this ugly code !!!
    StringTokenizer tokenizer = new StringTokenizer(theory, ",");
    ModSpecIdEl modTheory = new ModSpecIdEl();
    ArrayList theoryStrings = new ArrayList();
    while(tokenizer.hasMoreTokens()) {
      theoryStrings.add(tokenizer.nextToken()); 
    }
    String[] strings = (String[])theoryStrings.toArray(
                                             new String[theoryStrings.size()]);
    modTheory.setText(strings);
    
    ModSpecIdEl modProblem = new ModSpecIdEl();
    tokenizer = new StringTokenizer(problem, ",");
    ArrayList problemStrings = new ArrayList();
    while(tokenizer.hasMoreTokens()) {
      problemStrings.add(tokenizer.nextToken()); 
    }
    strings = (String[])problemStrings.toArray(new String[problemStrings.size()]);
    modProblem.setText(strings);

    ModSpecIdEl modMethod = new ModSpecIdEl();
    tokenizer = new StringTokenizer(method, ",");
    ArrayList methodStrings = new ArrayList();
    while(tokenizer.hasMoreTokens()) {
      methodStrings.add(tokenizer.nextToken()); 
    }
    strings = (String[])methodStrings.toArray(new String[methodStrings.size()]);
    modMethod.setText(strings);
    
    Formula[] formulasGiven = new Formula[1];
    formulasGiven[0] = new Formula();
    formulasGiven[0].setText(given);
    ModSpecFormulaListEl formulaListGiven = new ModSpecFormulaListEl();
    formulaListGiven.setFormulas_(formulasGiven);
    
    Formula[] formulasWhere = new Formula[1];
    formulasWhere[0] = new Formula();
    formulasWhere[0].setText(where);
    ModSpecFormulaListEl formulaListWhere = new ModSpecFormulaListEl();
    formulaListWhere.setFormulas_(formulasWhere);
    
    Formula[] formulasFind = new Formula[1];
    formulasFind[0] = new Formula();
    formulasFind[0].setText(find);
    ModSpecFormulaListEl formulaListFind = new ModSpecFormulaListEl();
    formulaListFind.setFormulas_(formulasFind);
    
    Formula[] formulasRelate = new Formula[1];
    formulasRelate[0] = new Formula();
    formulasRelate[0].setText(relate);
    ModSpecFormulaListEl formulaListRelate = new ModSpecFormulaListEl();
    formulaListRelate.setFormulas_(formulasRelate);
    
    ModSpec modSpec = new ModSpec();
    modSpec.setGiven(formulaListGiven);
    modSpec.setFind(formulaListFind);
    modSpec.setWhere(formulaListWhere);
    modSpec.setRelate(formulaListRelate);
    modSpec.setTheory(modTheory);
    modSpec.setProblem(modProblem);
    modSpec.setMethod(modMethod);
    
    ws.writeTestOutput("ModSpec: " + modSpec.getText());
    try {
      dialog.newCalculation(modSpec);
    } catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    // ws.setTreeTableModel();                                   
  };
  
  public static void main(String[] args) {
    WindowApplication windowApplication = new WindowApplication();    
  }
}
